// build a c++ interface to Gnuplot
// this stage is very low tech - write graph information to files
// and then run gnuplot on these files
//
// try to be sufficiently flexible to enable other
// interfaces to be built
//
// try to get principles right


#define WANT_STREAM
#define WANT_MATH

#include "gnuplot.h"

#include "newmatio.h"
#include <fstream.h>           // file io
#include "callplot.h"

// FILE SEPARATOR
//static char bs = '/';        // file name separator for unix
static char bs[] = "\\";       // file name separator for msdos & windows

// make a new alphabetic name
static String NewName()
{
   static int static_count;
   int count = static_count++;
   int c1 = count % 26;
   int c2 = (count / 26) % 26;
   int c3 = (count / (26*26)) % 26;
   char alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   return String("GP") + alphabet[c3] + alphabet[c2] + alphabet[c1];
}

// throw exception if P is null
static void CheckSpace(const void* P) { if (!P) Throw(Bad_alloc()); }


BaseSeries::BaseSeries(const String& title, const Matrix& data)
   : Title(title)
{
   // ignore series with no elements
   if (data.Nrows() == 0) { FileName = ""; return; }

   // make file name
   FileName = NewName();

   // write data to disk
   ofstream fout((TemporaryDirectory+bs+FileName).c_str());
   if (!fout) Throw(Runtime_error("File open fails\n"));
   int nrows = data.Nrows(); int ncols = data.Ncols();
   for (int i = 1; i <= nrows; i++)
   {
      if (data(i,1) < 0.99999E50)        // assume missing otherwise
         for (int j = 1; j <= ncols; j++)
            fout << " " << setprecision(15) << setw(20) << data(i, j);
      fout << endl;
   }
   fout << endl;
}

String LineSeries::Style() const
   { String S = String("lines ") + Colour; return S; }

BaseSeries* LineSeries::Clone() const
   { BaseSeries* bs = new LineSeries(*this); CheckSpace(bs); return bs; }


String ImpulseSeries::Style() const
   { String S = String("impulses ") + Colour; return S; }

BaseSeries* ImpulseSeries::Clone() const
   { BaseSeries* bs = new ImpulseSeries(*this); CheckSpace(bs); return bs; }

String StepSeries::Style() const
   { String S = String("steps ") + Colour; return S; }

BaseSeries* StepSeries::Clone() const
   { BaseSeries* bs = new StepSeries(*this); CheckSpace(bs); return bs; }

String BoxSeries::Style() const
   { String S = String("boxes ") + Colour; return S; }

BaseSeries* BoxSeries::Clone() const
   { BaseSeries* bs = new BoxSeries(*this); CheckSpace(bs); return bs; }

String PointSeries::Style() const
   { String S = String("points ") + Colour + String(" ") + Shape; return S; }

BaseSeries* PointSeries::Clone() const
   { BaseSeries* bs = new PointSeries(*this); CheckSpace(bs); return bs; }

String DotSeries::Style() const
   { String S = String("dots ") + Colour; return S; }

BaseSeries* DotSeries::Clone() const
   { BaseSeries* bs = new DotSeries(*this); CheckSpace(bs); return bs; }

LL_Series::LL_Series(LL_Series* next, const BaseSeries& bs)
   : Next(next), Series(bs.Clone()) {}

LL_Series::~LL_Series()
   { if (Series) delete Series;  if (Next) delete Next; }

Line::Line(Real x1, Real y1, Real x2, Real y2)
   { Location.ReSize(2,2); Location << x1 << y1 << x2 << y2; }

Line::Line(Real x1, Real y1, Real z1, Real x2, Real y2, Real z2)
   { Location.ReSize(2,3); Location << x1 << y1 << z1 << x2 << y2 << z2; }

Arrow::Arrow(Real x1, Real y1, Real x2, Real y2)
   { Location.ReSize(2,2); Location << x1 << y1 << x2 << y2; }

Arrow::Arrow(Real x1, Real y1, Real z1, Real x2, Real y2, Real z2)
   { Location.ReSize(2,3); Location << x1 << y1 << z1 << x2 << y2 << z2; }

Label::Label(const String& label, Real x, Real y, Justification j)
      : TheLabel(label), justify(j)
   { Location.ReSize(1,2); Location << x << y; }

Label::Label(const String& label, Real x, Real y, Real z, Justification j)
      : TheLabel(label), justify(j)
   { Location.ReSize(1,3); Location << x << y << z; }

void Line::PlotIt(Plot2& P) const
{
   P.PartCommand("set arrow from ");
   int n = Location.Ncols(); int i;
   P.PartCommand(ToString(Location(1,1)));
   for (i = 2; i <= n; i++)
      { P.PartCommand(","); P.PartCommand(ToString(Location(1,i))); }
   P.PartCommand(" to ");
   P.PartCommand(ToString(Location(2,1)));
   for (i = 2; i <= n; i++)
      { P.PartCommand(","); P.PartCommand(ToString(Location(2,i))); }
   P.PartCommand(" nohead\n");
}

void Arrow::PlotIt(Plot2& P) const
{
   P.PartCommand("set arrow from ");
   int n = Location.Ncols(); int i;
   P.PartCommand(ToString(Location(1,1)));
   for (i = 2; i <= n; i++)
      { P.PartCommand(","); P.PartCommand(ToString(Location(1,i))); }
   P.PartCommand(" to ");
   P.PartCommand(ToString(Location(2,1)));
   for (i = 2; i <= n; i++)
      { P.PartCommand(","); P.PartCommand(ToString(Location(2,i))); }
   P.PartCommand(" head\n");
}

void Label::PlotIt(Plot2& P) const
{
   P.PartCommand("set label \"");
   P.PartCommand(TheLabel);
   P.PartCommand("\" at ");
   int n = Location.Ncols(); int i;
   P.PartCommand(ToString(Location(1,1)));
   for (i = 2; i <= n; i++)
      { P.PartCommand(","); P.PartCommand(ToString(Location(1,i))); }
   switch (justify)
   {
   case left: P.PartCommand(" left\n"); break;
   case right: P.PartCommand(" right\n"); break;
   case centre: case center: P.PartCommand(" center\n"); break;
   default: P.PartCommand("\n"); break;
   }
}




Plot2& Plot2::operator<< (const BaseSeries& BS)
{
   if (BS.HasValues()) { LLS = new LL_Series(LLS,BS); CheckSpace(LLS); }
   return *this;
}

void Plot2::AddFunction(const String& function)
{
   if (Functions != "") Functions += ",";
   Functions += function;
   Functions += " \\\n";              // continuation character and newline
}

void Plot2::DoPlot()
{

#ifdef IMMEDIATE
   CommandFile = TemporaryDirectory+bs+NewName()+".gp";
   ofstream fout(CommandFile.c_str());
#else
   if (CommandFile=="") CommandFile = TemporaryDirectory+bs+NewName()+".gp";
   ofstream fout(CommandFile.c_str(), ios::app);
#endif
   if (!fout) Throw(Runtime_error("Command file open fails\n"));
   fout << "set title \"" << Title << "\"" << endl;

   // set the axes labels
   if (X_Axis.IsSet)
      fout << "set xlabel \"" << X_Axis.Title << "\"" << endl;
   else fout << "set xlabel" << endl;

   if (Y_Axis.IsSet)
      fout << "set ylabel \"" << Y_Axis.Title << "\"" << endl;
   else fout << "set ylabel" << endl;

   // set the axes scale type (linear or log)

   fout << "set nologscale" << endl;
   if (Y_Axis.IsSet && Y_Axis.IsLog) fout << "set logscale y" << endl;
   if (X_Axis.IsSet && X_Axis.IsLog) fout << "set logscale x" << endl;

   // do we want a zero-axis

   fout << "set nozeroaxis" << endl;
   if (Y_Axis.IsSet && Y_Axis.ZeroAxis) fout << "set yzeroaxis" << endl;
   if (X_Axis.IsSet && X_Axis.ZeroAxis) fout << "set xzeroaxis" << endl;

    // do we want the key (legend)
   if (WK) fout << "set key" << endl;
   else fout << "set nokey" << endl;

   // do axis ranges
   fout << "set autoscale" << endl;
   if (Y_Axis.IsSet && Y_Axis.Wrange)
      fout << "set yrange [" << Y_Axis.Min << ":" << Y_Axis.Max << "]"
	 << endl;
   if (X_Axis.IsSet && X_Axis.Wrange)
      fout << "set xrange [" << X_Axis.Min << ":" << X_Axis.Max << "]"
	 << endl;

   // do command strings
   fout << Commands;

   bool first = true;
   fout << "plot \\" << endl;
   if (Functions != "") { first = false; fout << Functions; }
   for (LL_Series* lls = LLS; lls; lls = lls->Next)
   {
      if (first) first = false;
      else fout << ", ";
      fout << "\"" << TemporaryDirectory << bs << lls->Series->FN();
      fout << "\" title \"" << lls->Series->T() << "\" ";
      fout << "with  " << lls->Series->Style() << " \\" << endl;
                                 // "\\" is the line continuation code
   }
   fout << endl;

#ifdef IMMEDIATE
   fout << "pause -1" << endl;
   fout.close();
   CallPlot(CommandFile);
#else
   fout << "pause -1 \"press <enter>\"" << endl;
   fout.close();
#endif

}

Plot2::~Plot2()
{ if (LLS) delete LLS; }        // this will delete the whole linked list

void Plot3::DoPlot()
{

#ifdef IMMEDIATE
   CommandFile = TemporaryDirectory+bs+NewName()+".gp";
   ofstream fout(CommandFile.c_str());
#else
   if (CommandFile=="") CommandFile = TemporaryDirectory+bs+NewName()+".gp";
   ofstream fout(CommandFile.c_str(), ios::app);
#endif
   if (!fout) Throw(Runtime_error("File open fails\n"));
   fout << "set title \"" << Title << "\"" << endl;
   fout << "set parametric" << endl;

   // set the axes labels
   if (X_Axis.IsSet)
      fout << "set xlabel \"" << X_Axis.Title << "\"" << endl;
   else fout << "set xlabel" << endl;

   if (Y_Axis.IsSet)
      fout << "set ylabel \"" << Y_Axis.Title << "\"" << endl;
   else fout << "set ylabel" << endl;

   if (Z_Axis.IsSet)
      fout << "set zlabel \"" << Z_Axis.Title << "\"" << endl;
   else fout << "set zlabel" << endl;

   // set the axes scale type (linear or log)

   fout << "set nologscale" << endl;
   if (Z_Axis.IsSet && Z_Axis.IsLog) fout << "set logscale z" << endl;
   if (Y_Axis.IsSet && Y_Axis.IsLog) fout << "set logscale y" << endl;
   if (X_Axis.IsSet && X_Axis.IsLog) fout << "set logscale x" << endl;

   // do we want a zero-axis

   fout << "set nozeroaxis" << endl;
   if (Z_Axis.IsSet && Z_Axis.ZeroAxis) fout << "set zzeroaxis" << endl;
   if (Y_Axis.IsSet && Y_Axis.ZeroAxis) fout << "set yzeroaxis" << endl;
   if (X_Axis.IsSet && X_Axis.ZeroAxis) fout << "set xzeroaxis" << endl;

    // do we want the key (legend)
   if (WK) fout << "set key" << endl;
   else fout << "set nokey" << endl;

   // do axis ranges
   fout << "set autoscale" << endl;
   if (Z_Axis.IsSet && Z_Axis.Wrange)
      fout << "set zrange [" << Z_Axis.Min << ":" << Z_Axis.Max << "]"
	 << endl;
   if (Y_Axis.IsSet && Y_Axis.Wrange)
      fout << "set yrange [" << Y_Axis.Min << ":" << Y_Axis.Max << "]"
	 << endl;
   if (X_Axis.IsSet && X_Axis.Wrange)
      fout << "set xrange [" << X_Axis.Min << ":" << X_Axis.Max << "]"
	 << endl;

   // do command strings
   fout << Commands;

   bool first = true;
   fout << "splot \\" << endl;
   if (Functions != "") { first = false; fout << Functions; }
   for (LL_Series* lls = LLS; lls; lls = lls->Next)
   {
      if (first) first = false;
      else fout << ", ";
      fout << "\"" << TemporaryDirectory << bs << lls->Series->FN();
      fout << "\" title \"" << lls->Series->T() << "\" ";
      fout << "with  " << lls->Series->Style() << " \\" << endl;
                                 // "\\" is the line continuation code
   }
   fout << endl;

#ifdef IMMEDIATE
   fout << "pause -1" << endl;
   fout.close();
   CallPlot(CommandFile);
#else
   fout << "pause -1 \"press <enter>\"" << endl;
   fout.close();
#endif

}



String TemporaryDirectory;
String CommandFile = "";




// convert integer or double to string

String ToString(int value)
// convert signed integer to string
{
   bool negative;
   if (value < 0) { value = -value; negative = true; }
   else if (value == 0) return String("0");
   else negative = false;
   char b[100]; b[99] = (char)0; int i = 99;
   while (value > 0)
   {
      int nv = value / 10; int rm = value - nv * 10;
      b[--i] = (char)(rm + '0'); value = nv;
      if (i<=0) return ("*****");
   }
   if (negative) b[--i] = '-';
   return String(b+i);
}

String ToString(long value)
// convert signed long to string
{
   bool negative;
   if (value < 0) { value = -value; negative = true; }
   else if (value == 0) return String("0");
   else negative = false;
   char b[100]; b[99] = (char)0; int i = 99;
   while (value > 0)
   {
      long nv = value / 10; long rm = value - nv * 10;
      b[--i] = (char)(rm + '0'); value = nv;
      if (i<=0) return ("*****");
   }
   if (negative) b[--i] = '-';
   return String(b+i);
}

String ToString(double value, int ndec)
// convert double to string, ndec = number of decimals
{
   bool negative;
   if (value < 0) { value = -value; negative = true; }
   else negative = false;
   if (value >= 1000000L) return ("*****");
   double p = pow(10,ndec); double i; double f = modf(value, &i);
   f = floor(f * p + 0.5);
   if (f >= p) i += 1.0; else f += p;
   long li = (long)i; long lf = (long)f; long lp = (long)p;
   if (!li && lf==lp) negative = false;
   String si = ToString(li); String sf = ToString(lf); sf[0] = '.';
   if (negative) return String("-")+si+sf; else return si+sf;
}

