// build a c++ interface to Gnuplot
// this stage is very low tech - write graph information to files
// and then run gnuplot on these files
//
// try to be sufficiently flexible to enable other
// interfaces to be built
//
// try to get principles right
                              
#ifndef GNUPLOT_LIB
#define GNUPLOT_LIB 0

#define IMMEDIATE              // to plot immediately

#include "include.h"
#include "newmat.h"            // my matrix package
#include "str.h"               // String class

// convert integer or double to string
String ToString(int i);

String ToString(long i);

String ToString(double f, int ndec = 4);



extern String TemporaryDirectory;
extern String CommandFile;

class BaseSeries
// the components such as LineSeries are derived from this
{
   String Title;
   String FileName;
public:
   BaseSeries(const String& title, const Matrix& data);
   BaseSeries(const BaseSeries& bs) : Title(bs.Title), FileName(bs.FileName) {}
   virtual ~BaseSeries() {}          // so deletes work
   String FN() const { return FileName; }
   String T() const { return Title; }
   virtual String Style() const = 0;
   virtual BaseSeries* Clone() const = 0;  // pointer to a copy of this class
   bool HasValues() const { return FileName != ""; }
};

class LineSeries : public BaseSeries
// plot a series as a line
{
   String Colour;
public:
   LineSeries(const String& title, const Matrix& data, const int colour)
      : BaseSeries(title, data), Colour(ToString(colour)) {}
   LineSeries(const String& title, const Matrix& data, const String& colour)
      : BaseSeries(title, data), Colour(colour) {}
   LineSeries(const LineSeries& bs) : BaseSeries(bs), Colour(bs.Colour) {}
   String Style() const;
   BaseSeries* Clone() const;         // pointer to a copy of this class
};

class ImpulseSeries : public BaseSeries
// plot a series as impulses
{
   String Colour;
public:
   ImpulseSeries(const String& title, const Matrix& data, const int colour)
      : BaseSeries(title, data), Colour(ToString(colour)) {}
   ImpulseSeries(const String& title, const Matrix& data, const String& colour)
      : BaseSeries(title, data), Colour(colour) {}
   ImpulseSeries(const ImpulseSeries& bs) : BaseSeries(bs), Colour(bs.Colour) {}
   String Style() const;
   BaseSeries* Clone() const;         // pointer to a copy of this class
};

class StepSeries : public BaseSeries
// plot a series as steps
{
   String Colour;
public:
   StepSeries(const String& title, const Matrix& data, const int colour)
      : BaseSeries(title, data), Colour(ToString(colour)) {}
   StepSeries(const String& title, const Matrix& data, const String& colour)
      : BaseSeries(title, data), Colour(colour) {}
   StepSeries(const StepSeries& bs) : BaseSeries(bs), Colour(bs.Colour) {}
   String Style() const;
   BaseSeries* Clone() const;         // pointer to a copy of this class
};

class BoxSeries : public BaseSeries
// plot a series as boxes
{
   String Colour;
public:
   BoxSeries(const String& title, const Matrix& data, const int colour)
      : BaseSeries(title, data), Colour(ToString(colour)) {}
   BoxSeries(const String& title, const Matrix& data, const String& colour)
      : BaseSeries(title, data), Colour(colour) {}
   BoxSeries(const BoxSeries& bs) : BaseSeries(bs), Colour(bs.Colour) {}
   String Style() const;
   BaseSeries* Clone() const;         // pointer to a copy of this class
};

class DotSeries : public BaseSeries
// plot a series as dots
{
   String Colour;
public:
   DotSeries(const String& title, const Matrix& data, const int colour)
      : BaseSeries(title, data), Colour(ToString(colour)) {}
   DotSeries(const String& title, const Matrix& data, const String& colour)
      : BaseSeries(title, data), Colour(colour) {}
   DotSeries(const DotSeries& bs)
      : BaseSeries(bs), Colour(bs.Colour) {}
   String Style() const;
   BaseSeries* Clone() const;         // pointer to a copy of this class
};

class PointSeries : public BaseSeries
// plot a series as points
{
   String Colour;
   String Shape;
public:
   PointSeries(const String& title, const Matrix& data,
      const int colour, const int shape)
      : BaseSeries(title, data), Colour(ToString(colour)),
         Shape(ToString(shape)) {}
   PointSeries(const String& title, const Matrix& data,
      const String& colour, const String& shape)
      : BaseSeries(title, data), Colour(colour), Shape(shape) {}
   PointSeries(const PointSeries& bs)
      : BaseSeries(bs), Colour(bs.Colour), Shape(bs.Shape) {}
   String Style() const;
   BaseSeries* Clone() const;        // pointer to a copy of this class
};

class LL_Series
// a linked list of Base_Series
{
   friend class Plot2;
   friend class Plot3;
   LL_Series* Next;             // points to next LL_Series
   BaseSeries* Series;          // points to a Series
   LL_Series(LL_Series* next, const BaseSeries& bs);
   ~LL_Series();                // delete this and subsequent LL_Series
};

class GraphObject
{
   virtual void PlotIt(Plot2&) const = 0;
protected:
   Matrix Location;
   friend class Plot2;
   friend class Plot3;
};

class Line : public GraphObject
{
   void PlotIt(Plot2&) const;
public:
   Line(Real x1, Real y1, Real x2, Real y2);
   Line(Real x1, Real y1, Real z1, Real x2, Real y2, Real z2);
   friend class Plot2;
   friend class Plot3;
};

class Arrow : public GraphObject
{
   void PlotIt(Plot2&) const;
public:
   Arrow(Real x1, Real y1, Real x2, Real y2);
   Arrow(Real x1, Real y1, Real z1, Real x2, Real y2, Real z2);
   friend class Plot2;
   friend class Plot3;
};

class Label : public GraphObject
{
public:
   enum Justification { left, right, centre, center };
private:
   void PlotIt(Plot2&) const;
   String TheLabel;
   Justification justify;
public:
   Label(const String& label, Real x, Real y, Justification j);
   Label(const String& label, Real x, Real y, Real z, Justification j);
   friend class Plot2;
   friend class Plot3;
};



class Axis
// axis of a graph
{
   String Title;
   bool IsLog;
   Real Max, Min;                  // range
   bool Wrange;                    // true if range set
   bool IsSet;                     // true if values are set
   bool ZeroAxis;                  // true if we want a zero axis
   Axis() : IsSet(false) {}        // values not set
public:
   Axis(const String& title) : Title(title), IsLog(false), Wrange(false),
      ZeroAxis(false), IsSet(true) {}
   void SetLog(bool il = true) { IsLog = il; }
   void SetRange(Real min, Real max) { Min=min; Max=max; Wrange = true; }
   void SetZeroAxis(bool za = true) { ZeroAxis = za; }
   friend class Plot2;
   friend class Plot3;
   // use default operator=
};

class Plot2
// two dimensional plot
{
protected:
   String Title;
   String Commands;
   String Functions;
   LL_Series* LLS;
   Axis X_Axis;
   Axis Y_Axis;
   bool WK;                       // true if we want the key (legend)
public:
   Plot2(const String& title)
      : Title(title), Commands(""), Functions(""), LLS(0), WK(true) {}
   Plot2& operator<< (const BaseSeries&);
                                  // now copies rather than passing a ref.
   Plot2& operator<< (const GraphObject& go)
      { go.PlotIt(*this); return *this; }
   void AddCommand(const String& command)
      { Commands += command; Commands += "\n"; }
   void PartCommand(const String& command) { Commands += command; }
   void Set(const String& command)
      { Commands += "set "; Commands += command; Commands += "\n"; }
   void AddFunction(const String& function);
   void Set_X_Axis(const Axis& xa) { X_Axis = xa; }
   void Set_Y_Axis(const Axis& ya) { Y_Axis = ya; }
                                  // copy rather than passing ref.
   void DoPlot();
   void WantKey(bool wk = true) { WK = wk; }
   ~Plot2();                      // destroy the linked list
};

class Plot3 : public Plot2
// three dimensional plot
{
protected:
   Axis Z_Axis;
public:
   Plot3(const String& title) : Plot2(title) {}
   void Set_Z_Axis(const Axis& za) { Z_Axis = za; }
   void DoPlot();
};




#endif

